﻿using System.Collections;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Interactivity;
using Infragistics.Windows.DockManager;
using Infragistics.Windows.DockManager.Events;

namespace WpfApplication1
{
    public class TabGroupPaneItemsSourceBehavior : Behavior<TabGroupPane>
    {
        public static readonly DependencyProperty HeaderPathProperty;
        public static readonly DependencyProperty HeaderTemplateProperty;
        public static readonly DependencyProperty ItemsSourceProperty;
        public static readonly DependencyProperty ItemTemplateProperty;
        public static readonly DependencyProperty ContainerStyleProperty;
        public static readonly DependencyProperty TabClosedCommandProperty;

        static TabGroupPaneItemsSourceBehavior()
        {
            HeaderPathProperty = DependencyProperty.Register("HeaderPath", typeof(string), typeof(TabGroupPaneItemsSourceBehavior));
            HeaderTemplateProperty = DependencyProperty.Register("HeaderTemplate", typeof(DataTemplate), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null));
            ItemsSourceProperty = DependencyProperty.Register("ItemsSource", typeof(IList), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null, OnItemsSourcePropertyChanged));
            ItemTemplateProperty = DependencyProperty.Register("ItemTemplate", typeof(DataTemplate), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null));
            ContainerStyleProperty = DependencyProperty.Register("ContainerStyle", typeof(Style), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null));
            TabClosedCommandProperty = DependencyProperty.Register("TabClosedCommand", typeof(ICommand), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null));
        }

        #region Properties

        public string HeaderPath
        {
            get { return (string)GetValue(HeaderPathProperty); }
            set { SetValue(HeaderPathProperty, value); }
        }

        public DataTemplate HeaderTemplate
        {
            get { return (DataTemplate)GetValue(HeaderTemplateProperty); }
            set { SetValue(HeaderTemplateProperty, value); }
        }

        public IList ItemsSource
        {
            get
            {
                return (IList)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        public DataTemplate ItemTemplate
        {
            get { return (DataTemplate)GetValue(ItemTemplateProperty); }
            set { SetValue(ItemTemplateProperty, value); }
        }

        public Style ContainerStyle
        {
            get { return (Style)GetValue(ContainerStyleProperty); }
            set { SetValue(ContainerStyleProperty, value); }
        }

        public ICommand TabClosedCommand
        {
            get { return (ICommand)GetValue(TabClosedCommandProperty); }
            set { SetValue(TabClosedCommandProperty, value); }
        }

        #endregion

        #region Protected Methods

        protected override void OnAttached()
        {
            OnItemsSourcePropertyChanged(null, ItemsSource);

            base.OnAttached();
        }

        protected override void OnDetaching()
        {
            OnItemsSourcePropertyChanged(ItemsSource, null);

            base.OnDetaching();
        }

        #endregion

        #region Private Methods

        private static void OnItemsSourcePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var behavior = d as TabGroupPaneItemsSourceBehavior;
            if (behavior != null && behavior.AssociatedObject != null)
            {
                behavior.OnItemsSourcePropertyChanged((IEnumerable)e.OldValue, (IEnumerable)e.NewValue);
            }
        }

        private void OnItemsSourcePropertyChanged(IEnumerable oldValue, IEnumerable newValue)
        {
            AssociatedObject.Items.Clear();

            if (oldValue != null)
            {
                var oldCollectionChanged = oldValue as INotifyCollectionChanged;
                if (oldCollectionChanged != null)
                {
                    oldCollectionChanged.CollectionChanged -= OnSourceCollectionChanged;
                }
            }

            if (newValue != null)
            {
                var collectionChanged = newValue as INotifyCollectionChanged;
                if (collectionChanged != null)
                {
                    collectionChanged.CollectionChanged += OnSourceCollectionChanged;
                }

                foreach (var item in newValue)
                {
                    var contentPane = PrepareContainerForItem(item);
                    AssociatedObject.Items.Add(contentPane);
                }
            }
        }

        private void OnSourceCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                var dockManager = XamDockManager.GetDockManager(AssociatedObject);
                if (dockManager != null)
                {
                    var contentPanes = dockManager.GetPanes(PaneNavigationOrder.VisibleOrder);
                    foreach (var contentPane in contentPanes)
                    {
                        var dc = contentPane.DataContext;
                        if (dc != null && e.OldItems.Contains(dc))
                        {
                            contentPane.ExecuteCommand(ContentPaneCommands.Close);
                        }
                    }
                }
            }
            else if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (var item in e.NewItems)
                {
                    ContentPane contentPane = PrepareContainerForItem(item);
                    if (contentPane != null)
                    {
                        AssociatedObject.Items.Add(contentPane);
                        AssociatedObject.SelectedItem = contentPane;
                    }
                }
            }
        }

        private ContentPane PrepareContainerForItem(object item)
        {
            var container = new ContentPane
            {
                Content = item,
                DataContext = item
            };

            if (ContainerStyle != null)
            {
                container.Style = ContainerStyle;
            }

            if (HeaderTemplate != null)
            {
                container.HeaderTemplate = HeaderTemplate;
            }

            if (ItemTemplate != null)
            {
                container.ContentTemplate = ItemTemplate;
            }

            container.CloseAction = PaneCloseAction.RemovePane;
            container.Closed += OnContainerClosed;
            CreateBindings(item, container);

            return container;
        }

        private void OnContainerClosed(object sender, PaneClosedEventArgs e)
        {
            var contentPane = sender as ContentPane;
            if (contentPane != null)
            {
                contentPane.Closed -= OnContainerClosed;

                var item = contentPane.DataContext;

                RemoveBindings(contentPane);

                if (ItemsSource != null)
                {
                    ItemsSource.Remove(item);
                }

                if (TabClosedCommand != null && TabClosedCommand.CanExecute(item))
                {
                    TabClosedCommand.Execute(item);
                }
            }
        }

        private void CreateBindings(object item, FrameworkElement contentPane)
        {
            if (!string.IsNullOrWhiteSpace(HeaderPath))
            {
                var headerBinding = new Binding(HeaderPath) { Source = item };
                contentPane.SetBinding(HeaderedContentControl.HeaderProperty, headerBinding);
            }
        }

        private static void RemoveBindings(DependencyObject contentPane)
        {
            BindingOperations.ClearBinding(contentPane, HeaderedContentControl.HeaderProperty);
        }

        #endregion
    }
}
